#
# HocusPocus
#
# Copyright (C)  2008 and onwards Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# python imports
# TODO: explore alsaaudio package once it is ready
# for package python-alsaaudio
# import alsaaudio
import logging
import re
# local imports
import utils

AMIXER = "/usr/bin/amixer"
ASOUNDCONF = "/usr/bin/asoundconf"

COMMAND = {
    'set-volume':
        [AMIXER, '-c', '%(card)s', '--', 'sset', '%(dev)s', 'playback', '%(vol)s'],
    'default-card':
        [ASOUNDCONF, 'set-default-card', '%(card)s'],
    'reset-card':
        [ASOUNDCONF, 'reset-default-card'],
    'get-volume':
        [AMIXER, "-c", "%(card)s", "--",  "sget", "%(dev)s"],
    'get-devices':
        [AMIXER, "-c", "%(card)s", "--",  "scontrols"]
    }

def RunCommand(name, **kwd):
    comm = COMMAND.get(name, None)
    if not comm:
        logging.error("bad command %s", name)
        return -1, ""

    return utils.RunCommand([c % kwd for c in comm])

# Example
# 1 [PCM2702        ]: USB-Audio - Burr-Brown Japan PCM2702

RE_CARD = re.compile(r"^\s+(\d+)\s+\[([^\]]+)\]:\s+(.+)$")

def GetCards():
    cards = []
    for line in  open("/proc/asound/cards"):
        match = RE_CARD.search(line)
        if match:
            cards.append((match.group(1),match.group(3)))
    return cards

RE_DEVICE =  re.compile(r"^.*['](.+)['],[0-9]+$")

def GetDevices(card):
    devices = {}
    result, data = RunCommand("get-devices", card=card)
    for line in data.split("\n"):
        match = RE_DEVICE.search(line)
        if match:
            devices[match.group(1)] = None
    return devices.keys()


def IncVolume(card, dev, value):
    if value < 0:
        value = "%fdB-" % (-1.0 * value)
    else:
        value = "%fdB+" % (1.0 * value)
    RunCommand("set-volume", card=card, dev=dev, vol=value)

# Example
#  Front Left: 104 [81%] [-24.00dB] Playback [on]
RE_VOL = re.compile(r"\[([0-9]+)[%]\].*\[(on|off)\]")


def GetVolume(card, dev):
    result, data = RunCommand("get-volume", card=card, dev=dev)
    match = RE_VOL.search(data)
    if match:
        return match.group(1), match.group(2)
    else:
        return "unknown", "unknown"

def SetMute(card, device):
    pass

class Handler:

    def __init__(self, util, config):
        self._util = util
        self._config = self._util.LoadConfig("alsa.conf")
        return

    def RunCommand(self, name, d):
        comm = self._config.commands.get(name, None)
        if not comm:
            logging.error("unknown command %s", name)
            return -1, ""
        comm = comm % d
        return utils.RunCommand(comm.split())

    def ConfList(self):
        result, data = self.RunCommand("conf-list", {})
        prolog, stuff = data.split(":",1)
        return stuff.split()

    def Handler(self, req, path, args):
        cmd = args.get("cmd", "")
        logging.info("handler alsa: %s", cmd)
        if cmd in ["set-volume", "reset-card", "default-card"]:
            RunCommand(cmd, **args)
        else:
            logging.error("unknown command %s", cmd)

        data = self._util.GetDefaultDict()
        devices = []
        for card, name in self._config.GetCards():
            obj = utils.Datablob()
            obj.card = utils.EscapeContents(card)
            obj.name = utils.EscapeContents(name)
            obj.devices = ""
            obj.volume = ""
            devices.append(obj)
            for device in GetDevices(card):
                volume, status = GetVolume(card, device)
                obj = utils.Datablob()
                obj.card = utils.EscapeContents(card)
                obj.name = ""
                obj.device = utils.EscapeContents(device)
                obj.volume = volume
                obj.url = "alsa?card=%s&amp;dev=%s" % (card, device)
                obj.status = status
                devices.append(obj)


        data['devices'] = devices
        data['plugin-name'] = "Alsa Sound"
        self._util.CompleteRequest(req,
                                   data,
                                   self._config.template)
        return
