#
# HocusPocus
#
# Copyright (C) 2008 and onwards Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


import logging
import os
import urllib

import utils

class KeyVal:
    def __init__(self, key, val):
        self.key = key
        self.val = val

class Handler:

    def __init__(self, util, config):
        self._util = util
        self._config = self._util.LoadConfig("fileviewer.conf")
        return

    def RunCommand(self, name):
        comm = self._config.GetCommand(name)
        if not comm:
            logging.error("unknown command %s", name)
            return -1, ""
        return utils.RunCommand(comm)


    def Handler(self, req, path, args):
        cmd = utils.EscapeContents(args.get("cmd", ""))
        refresh = args.get("refresh", str(self._config.refresh))
        refresh = int(utils.EscapeContents(refresh))

        logging.info("handler fileviewer: %s  refresh %d", cmd, refresh)
        contents = ""
        if cmd:
            try:
              result, contents = self.RunCommand(cmd)
            except Exception, err:
              contents = utils.EscapeContents(str(err))

        data = self._util.GetDefaultDict()
        data['contents'] = contents
        data['cmd'] = cmd
        data['plugin-name'] = "System Viewer"
        data['refresh-time'] = str(refresh)

        data['refresh-time'] = refresh
        data['refresh-url'] = "fileviewer?cmd=%s&amp;refresh=%d" % (cmd, refresh)
        data['refreshs'] = utils.MakeBlobRefreshList(
            "fileviewer?refresh=%d&amp;cmd=" + cmd,
            self._config.refreshs,
            refresh)

        data['items'] = self._config.GetLabels()

        self._util.CompleteRequest(req,
                                   data,
                                   self._config.template)
        return

