#
# HocusPocus Media Plugin
#
# Copyright (C)  2008 and onwards Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# python imports
import logging
import urllib

# local imports
import utils

def MakeUrl(cols, genre=None, filename=None):
   url = "media?cols=%d" % cols
   if genre:
      url += "&amp;genre=" + genre

   if filename:
      url += "&amp;loc=" + urllib.quote_plus(filename)

   return url


class Handler:

    def __init__(self, util, config):
        self._util = util
        self._config = self._util.LoadConfig("media.conf")
        return

    def PlayMedia(self, genre, filename):
        logging.info("launching %s for %s", type, filename)
        self._config.PlayMedia(genre, filename)
        return

    def Handler(self, req, path, args):
        if "genre" in args and args["genre"] in self._config.GetGenres():
            self.HandlerMedia(req, path, args)
        else:
            self.HandlerGenre(req, path, args)

    def HandlerGenre(self, req, path, args):
        logging.info("handle genre:")
        cols = int(utils.EscapeContents(args.get('cols', str(self._config.cols))))
        def make_obj(genre):
            blob =  utils.Datablob()
            blob.url = MakeUrl(cols, genre)
            blob.name = utils.EscapeContents(genre)
            return blob

        data = self._util.GetDefaultDict()
        data['genres'] = [make_obj(x) for x in self._config.GetGenres()]
        data['plugin-name'] = "Media"
        data['cols'] = cols
        data['columns'] = self.MakeColumnBlobs()
        data['thisurl'] = "media"
        self._util.CompleteRequest(req,
                                   data,
                                   self._config.template_genre)
        return

    def MakeColumnBlobs(self, genre=None):
        columns = []
        for col in [1, 2, 3]:
            v = utils.Datablob()
            v.url = MakeUrl(col, genre)
            v.tag = "%d-Columns" % col
            columns.append(v)
        return columns

    def HandlerMedia(self, req, path, args):
        genre = utils.EscapeContents(args["genre"])
        cols = int(utils.EscapeContents(args.get('cols', str(self._config.cols))))
        loc = utils.EscapeContents(args.get('loc', ''))
        logging.info("handle media: %s", genre)
        if loc:
            self.PlayMedia(genre, urllib.unquote_plus(loc))

        def make_obj(name, loc, genre, cols):
            blob = utils.Datablob()
            blob.url = MakeUrl(cols, genre, loc)
            blob.name = utils.EscapeContents(name)
            return blob

        data = self._util.GetDefaultDict()
        data["plugin-name"] = genre
        data['thisurl'] = MakeUrl(cols, genre)
        data['cols'] = str(cols)
        data['columns'] = self.MakeColumnBlobs(genre)
        data['media'] = [make_obj(name, loc, genre, cols)
                         for name, loc in self._config.GetMediaForGenre(genre)]
        self._util.CompleteRequest(req,
                                   data,
                                   self._config.template_media)
        return
