#
# HocusPocus
#
# Copyright (C)  2008 and onwards Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# python imports
# TODO: explore alsaaudio package once it is ready
# for package python-alsaaudio
# import alsaaudio
import logging
import os
import re
# local imports
import utils


PACTL = "/usr/bin/pactl"


def HavePrequisites():
    return os.path.exists(PACTL)

def Info():
    comm = "%s %s" % (PACTL, "stat")
    return utils.RunCommand(comm.split())

def List():
    comm = "%s %s" % (PACTL, "list")
    return utils.RunCommand(comm.split())


def Reset():
    comm = "%s %s" % (PACTL, "exit")
    return utils.RunCommand(comm.split())


class Sink:
    def __init__(self, no, lines):
        self.no = no
        for l in lines:
            l = l.strip()
            if l.startswith("Volume:"):
                self.volume = l
            elif l.startswith("Description:"):
                self.description = l
            elif l.startswith("Mute:"):
                self.mute = l
            elif l.startswith("State:"):
                self.state = l

    def MakeBlob(self):
         obj = utils.Datablob()
         obj.no = self.no
         obj.volume = self.volume
         obj.state = self.state
         obj.mute = self.mute
         obj.description = self.description
         return obj


class Input:
    def __init__(self, no, lines):
        self.no = no
        for l in lines:
            l = l.strip()
            if l.startswith("Volume:"):
                self.volume = l
            elif l.startswith("Sink:"):
                self.sink = l
            elif l.startswith("application.name = "):
                self.app_name = l

    def MakeBlob(self):
         obj = utils.Datablob()
         obj.no = self.no
         obj.volume = self.volume
         obj.sink = self.sink
         obj.app_name = self.app_name
         return obj

def GetSinksAndInputs():
    sinks = []
    inputs = []
    ret, out = List()
    sections = out.split("\n\n")
    logging.info("found %d sections in pulse list", len(sections))
    for s in sections:
        lines =  s.split("\n")
        header = lines.pop(0)
        section_type, no =  header.rsplit(None, 1)
        if section_type == "Sink":
            sinks.append(Sink(no, lines))
        elif section_type == "Sink Input":
            inputs.append(Input(no, lines))
    return sinks, inputs

class Handler:

    def __init__(self, util, config):
        self._util = util
        self._config = self._util.LoadConfig("pulse.conf")
        return


    def Handler(self, req, path, args):
        if not HavePrequisites():
            self._util.ErrorPage(req, "pulse tools not installed")
            return
        cmd = args.get("cmd", "")
#        card = args.get("card", "")
#        vol = args.get("vol", "")
#        dev = args.get("dev", "")
        logging.info("handler pulse: %s", cmd)
        if cmd == "reset":
            Reset()
        else:
            logging.error("unknown command [%s]", cmd)
        data = self._util.GetDefaultDict()

        sinks, inputs = GetSinksAndInputs()

        data['info'] = Info()[1]
        data['plugin-name'] = "Pulse Audio"
        data['sinks'] = [s.MakeBlob() for s in sinks]
        data['inputs'] = [s.MakeBlob() for s in inputs]
        self._util.CompleteRequest(req,
                                   data,
                                   self._config.template)
        return
