#
# HocusPocus Quodlibet Plugin
#
# Copyright (C)  2008 and onwards Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os
import time

import utils

def GetCurrentSong():
   try:
       return open(os.getenv("HOME") + "/.quodlibet/current")
   except:  
       return None
   
class Handler:

    def __init__(self, util, config):
        self._util = util
        self._config = self._util.LoadConfig("quodlibet.conf")
        return

    def RunAudTool(self, name):
       comm = self._config.commands.get(name, None)
       if not comm:
           logging.error("bad command %s", name)
           return -1, ""
       else:
           return utils.RunCommand(comm)

    def GetSongTitle(self):
        d = {}
        # sometime we are faster than QL with getting to .quodlibet/current
        fp = GetCurrentSong()
        if not fp:
            time.sleep(1)
            fp = GetCurrentSong()
        if not fp:
            return "quodlibet appears to not be running"
        for line in fp:
            key, val = line.split("=", 1)
            d[key] = val
        fp.close()
        title = "%s: %s (%s) rating: %s" % (d.get("artist", ""),
                                            d.get("title", ""),
                                            d.get("album", ""),
                                            d.get("~#rating", ""))

        return utils.EscapeContents(title)


    def Handler(self, req, path, args):
        cmd = args.get("cmd", "")
        logging.info("handler audacious: %s", cmd)
        if cmd:
            self.RunAudTool(cmd)

        song = utils.Datablob()
        title = self.GetSongTitle()
        song.title = title
        song.dur = 0
        song.pos = 0

        norm_dur = 0
        if int(song.dur):
            norm_dur = 20 * int(song.pos) / int(song.dur)

        data= self._util.GetDefaultDict()
        data['song'] =  song
        data['refresh-time'] = str(self._config.refresh)
        data['refresh-url'] = "quodlibet"
        data['plugin-name'] = "Quodlibet"
        self._util.CompleteRequest(req,
                                   data,
                                   self._config.template)
        return
