# ======================================================================
# Copyright (C) 2008 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

# python imports
import logging
import time

# local imports
import utils
######################################################################
##
######################################################################

def MakeUserBlob(ip, timestamp, url):
    d = utils.Datablob()
    d.ip = ip
    d.url = url
    d.unquoted_url = url
    d.time = time.strftime("%B %d %I:%M:%S %p", time.localtime(timestamp))
    return d

def RenderUserLog(USER_LOG):
    lst = []
    for i in range(len(USER_LOG) - 1, -1, -1):
        ip, timestamp, url = USER_LOG[i]
        lst.append(MakeUserBlob(ip, timestamp, url))
    return lst

######################################################################

def MakeStatBlob(family, name, count):
    d = utils.Datablob()
    d.family = family
    d.name = name
    d.count = count
    return d

def RenderStats(COUNT_STATS):
    lst = []
    keys = sorted(COUNT_STATS.keys())
    last_family = None
    for k in keys:
        if k[0] != last_family:
            lst.append(MakeStatBlob(k[0], "", ""))
            last_family = k[0]
        lst.append(MakeStatBlob("", k[1], COUNT_STATS[k]))
    return lst

######################################################################

def MakeLogBlob(timestamp, kind, filename, message):
    d = utils.Datablob()
    d.timestamp = timestamp
    d.kind = kind
    d.filename = utils.EscapeContents(filename)
    d.message = utils.EscapeContents(message)
    return d

def RenderLogs(filename):
    lst = []
    try:
        lines = open(filename, "r").read().split("\n")
    except:
        logging.error("no log file %s", filename)
        lines = []

    for line in lines:
        token = line.split(":", 3)
        if len(token) != 4:
           logging.warning("bad line: %s", line)
           continue
        d = MakeLogBlob("",
                        token[0],
                        token[1] + ":" + token[2],
                        token[3])
        lst.append(d)
    return lst[-500:]

######################################################################
#
######################################################################
class Handler:

    def __init__(self, util, config):
        self._util = util
        self._config = self._util.LoadConfig("stats.conf")    
        return


    def Handler(self, req, path, args):
        data = self._util.GetDefaultDict()
        data['plugin-name'] = "Stats and Logs"
        data['users'] =  RenderUserLog(self._util._user_log)
        data['stats'] =  RenderStats(self._util._stats_count)
        data['logs'] = RenderLogs(self._util.log_file)
        self._util.CompleteRequest(req, data, self._config.template)
        return









