#
# HocusPocus
#
# Copyright (C)  2008 and onwards Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import socket

import utils

class Handler:

    def __init__(self, util, config):
        self._util = util
        self._config = self._util.LoadConfig("vlc.conf")
        return

    def MakeSocketRequest(self, name):
       comm = self._config.commands.get(name, None)
       if not comm:
           logging.error("bad command %s", name)
           return ""
       s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
       s.connect(self._config.server)
       s.settimeout(0.1)
       logging.info("command is %s", repr(comm))
       s.send(comm)
       data = ""
       while 1:
           try:
               data += s.recv(1024)
           except:
               break
       s.close()
       return data

    def Handler(self, req, path, args):
        cmd = args.get("cmd", "")
        cmd2 = args.get("cmd2", "")
        logging.info("handler vlc: %s", cmd)
        if cmd:
            try:
                self.MakeSocketRequest(cmd)
            except Exception, err:
                logging.info(str(err))

        if cmd2:
            cmd2 = self._config.commands2.get(cmd2, None)
            try:
                utils.RunCommand(cmd2)
            except:
                pass
        try:
            status = self.MakeSocketRequest('status')
        except Exception, err:
            logging.info(str(err))
            status = "vlc appears to not be running"

        data = self._util.GetDefaultDict()
        data['status'] = status
        data['plugin-name'] = "VLC"
        self._util.CompleteRequest(req,
                                   data,
                                   self._config.template)
        return
