#
# HocusPocus Webcam Plugin
#
# Copyright (C)  2008 and onwards Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import base64
import logging
import os

import utils

class KeyVal:
    def __init__(self, key, val):
        self.key = key
        self.val = val


class Handler:

    def __init__(self, util, config):
        self._util = util
        self._config = self._util.LoadConfig("webcam.conf")
        return

    def GrapSnapshot(self, command, filename):
        try:
            os.remove(filename)
        except:
            pass
        command = command % filename
        result, output = utils.RunCommand(command.split())
        if result:
            logging.error("bad exit code %s %s", str(result), str(output))
        try:
            return open(filename).read()
        except:
            return None

    def Handler(self, req, path, args):
        sources = self._config.GetSources()
        device = utils.EscapeContents(args.get("device", "video0"))
        logging.info("handler webcam")
        filename = "/tmp/xxxx.jpeg"
        refresh = int(utils.EscapeContents(args.get("refresh",
                                                    str(self._config.refresh))))

        images = []
        if device in sources:
            img_data = self.GrapSnapshot(sources[device], filename)
            if img_data:
                images.append(KeyVal(device, base64.b64encode(img_data)))

        data = self._util.GetDefaultDict()
        data['plugin-name'] = "Webcam"
        data['refreshs'] = utils.MakeBlobRefreshList("webcam?refresh=%s&amp;device=" +
                                                     device,
                                                     self._config.refreshs,
                                                     refresh)
        data['refresh-time'] = str(refresh)
        data['device'] = device
        devices =  sources.keys()
        devices.sort()
        data['devices'] = devices
        data['images'] = images
        self._util.CompleteRequest(req,
                                   data,
                                   template = self._config.template)
        return
