#
# HocusPocus
#
# Copyright (C)  2008 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import base64

class Authenticator(object):

    def __init__(self, machines, users):
        """
        """
        self._machines = machines
        self._users = users
        return

    def AuthenticateMachine(self, request):
        client, dummy = request.client_address
        if client in self._machines:
            return ("machine", client)
        else:
            return None

    def GetUserAndPassword(self, request):
        auth = request.headers.getheader('Authorization')
        if not auth:
            return None, None

        auth_tokens = auth.split()

        if auth_tokens[0].lower() != 'basic':
            logging.warning("wrong authentication type")
            return None, None

        return  base64.decodestring(auth_tokens[-1]).split(':')


    def AuthenticateUser(self, request):
        if not self._users:
            return None
        user, password = self.GetUserAndPassword(request)
        # empty password not accepted!
        if not user or not password:
            return None
        if user not in self._users:
            return None
        if self._users[user][0] != password:
            return None
        return ("user", user)

    def Authenticate(self, request):
        """Authenticate a request:
        * certain machines (IP addresses really) are permitted
        * otherwise we have to check user and passwd
        """
        t = self.AuthenticateMachine(request)
        if t: return t

        t = self.AuthenticateUser(request)
        if t: return t

        return None
