#!/usr/bin/env python
#
# HocusPocus
#
# Copyright (C) Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


# python import
import SocketServer
import BaseHTTPServer
import getopt
import glob
import imp
import logging
import os
import re
import select
import signal
import socket
import sys
import time
import urllib
import urlparse


#local imports
import authentication

######################################################################
#
######################################################################

TOPLEVEL="""
<html>
<head>
<style type="text/css">
body {
   background: white;
   color: black;
   font-family: sans-serif;
 }


a:link {
    text-decoration: none;
    color: black;
}

a:visited {
    text-decoration: none;
    color: black;
}

a:hover {
    text-decoration: none;
    color: yellow;
}

h1.home {
   font-size: 30pt;
   background: #86bcd6;
   // background: #dbebf3;
   padding: 0.2em 0.2em 0.2em 0.2em;
   margin: 0.2em 0em 0.2em 0em;
    -moz-border-radius: 10px;
    border-radius: 10px;
    -webkit-border-radius: 10px;
}

h1.room {
   font-size: 28pt;
   background: #dbebf3;
   padding: 0.2em 0.2em 0.2em 0.2em;
   margin: 0.2em 0em 0.2em 0em;
    -moz-border-radius: 10px;
    border-radius: 10px;
    -webkit-border-radius: 10px;
}
</style>
</head>
<body>
<a href=update><h1 class=home>%s</h1></a>
%s
</body>
</html>
"""

######################################################################
#
######################################################################

class _datablob:
    def __init__(self, **args):
        self.__dict__.update(args)


def LoadConfig(fname):
    desc =  ('.py', 'U', 1)
    dummy, name = os.path.split(fname)
    name, dummy = os.path.splitext(name)
    logging.info("loading config %s %s", fname, name)
    mod =  imp.load_module(name, open(fname), fname, desc)
    return mod


def FindAvailableServers(servers):
    socket.setdefaulttimeout(1)
    lst = {}
    for host, port in servers:
        logging.info("trying server: %s:%d", host, port)
        try:
            data = urllib.urlopen("http://%s:%d/id" % (host, port))
            id = data.read().strip()
            logging.info("active: %s", id)
            while urllib.quote_plus(id) in lst:
                logging.warning("found duplicate [%s]", id)
                id += ".dup"
            lst[urllib.quote_plus(id)] = (host, port)
        except:
            logging.info("inactive")
    socket.setdefaulttimeout(10)
    return lst


class ProxyHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    # redirect all references to the original server to the proxy
    # this is currently accomplished by rewriting the "base" tag
    def RewriteHtml(self, html, service):
        proxy = "/%s/" % (service)
        return html.replace('<base href="/" />',
                            '<base href="%s" />' % proxy)

    def ServeMainMenuPage(self, config, servers):
        self.send_response(200)
        self.send_header('Content-Type', 'text/html;');
        self.end_headers()
        lines = []
        for k in servers:
            lines.append("<a href='/%s/'><h1 class=room>%s</h1></a>" %
                         (k, urllib.unquote_plus(k)))
        self.wfile.write(TOPLEVEL % (config.title, "\n".join(lines)))
        return


    def ServeFavicon(self, config, servers):
        logging.info("server favicon from %s", config.favicon)
        self.send_response(200)
        self.send_header('Content-Type', 'image/x-icon;');
        self.end_headers()
        icon = open(config.favicon).read()
        self.wfile.write(icon)
        return

    def ProxyOneRequest(self, host, url, name):
        soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        num_bytes = 0
        try:
            soc.connect(host)
            cmd = "%s %s %s\r\n" % (self.command, url, self.request_version)
            soc.send(cmd)

            self.headers['Connection'] = 'close'
            for key_val in self.headers.items():
                soc.send("%s: %s\r\n" % key_val)
            soc.send("\r\n")

            num_bytes = 0
            if 1:
                while 1:
                    data = soc.recv(10 * 1024)
                    if not data: break
                    data = self.RewriteHtml(data, name)
                    num_bytes += len(data)
                    self.wfile.write(data)
            else:
                self._read_write(soc)
            logging.info("request completed %d", num_bytes)
        except Exception, err:
            logging.error("proxy problem after %d bytes: %s",
                          num_bytes, str(err))
            if num_bytes == 0:
                self.ServeMainMenuPage(self.server._config,
                                       self.server._servers)
        finally:
            logging.info("proxy complete")
            soc.close()
            self.connection.close()
        return

    def SendAuthenticationRequest(self, data):
        realm='hocuspocus'
        self.send_response(401)
        self.send_header('WWW-Authenticate', 'Basic realm="%s"' % realm)
        self.send_header('Content-Type', 'text/html;');
        self.send_header('Connection', 'close');
        self.end_headers()
        self.wfile.write(data)
        return

    def do_GET(self):
        logging.info("incoming path [%s] [%s]", self.path, self.client_address)
        config = self.server._config
        authenticator = authentication.Authenticator(config.auth_machines,
                                                     config.auth_users)
        t = authenticator.Authenticate(self)
        if not t:
            if self.server._config.auth_users:
                logging.info("authentication request")
                self.SendAuthenticationRequest("Authentication Required")
            return

        scm, host, path, params, query, fragment = urlparse.urlparse(self.path)

        if path == "/update":
            self.server.UpdateServerList()
            self.ServeMainMenuPage(config, self.server._servers)
            return

        if path == "/favicon.ico":
            self.ServeFavicon(config, self.server._servers)
            return

        if not path or path == "/":
            self.ServeMainMenuPage(config, self.server._servers)
            return

        tokens = [t for t in path.split("/") if t]
        name = tokens[0]
        logging.info("proxy request for: %s", name)
        server = self.server.MapNameToServer(name)
        if not server:
            self.send_error(404, "bad/unavailable server %s" % name)
            return

        path = path[len(name)+1:]
        url = urlparse.urlunparse(('', '', path, params, query, fragment))

        logging.info("rewritten request [%s] for %s:%d",
                     url, server[0], server[1])
        self.ProxyOneRequest(server, url, name)
        return

    def _read_write(self, soc, max_idling=20):
        iw = [self.connection, soc]
        ow = []
        count = 0
        while 1:
            count += 1
            (ins, _, exs) = select.select(iw, ow, iw, 3)
            if exs: break
            if ins:
                for i in ins:
                    if i is soc:
                        out = self.connection
                    else:
                        out = soc
                    data = i.recv(8192)
                    if data:
                        out.send(data)
                        count = 0
            else:
                print "\t" "idle", count
            if count == max_idling: break

class Server(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):

    def __init__(self, config):
        self._config = config
        port = config.port
        self.UpdateServerList()
        logging.info("*" * 70)
        logging.info("starting server on port %d", port)
        logging.info("*" * 70)
        logging.info("proxy ready, point your browser at   "
                     "http://localhost:%d", port)
        logging.info("*" * 70)
        SocketServer.TCPServer.__init__(self, ("", port), ProxyHandler)


    def MapNameToServer(self, name):
        return self._servers.get(name, None)

    def UpdateServerList(self):
        logging.info("updating available server list")
        self._servers = FindAvailableServers(self._config.potential_servers)
        return

def ParseCommandline(argv):
    defaults = _datablob()
    defaults.debug_level = logging.INFO
    defaults.config_file = "service_manager.conf"

    try:
        opts, args = getopt.getopt(argv[1:], "f:d:")
    except getopt.GetoptError, err:
        return  str(err), defaults

    for o, a in opts:
        if o == "-d":
            defaults.defaults = int(a)
        elif o == "-f":
            defaults.config_file = a
        else:
            assert False, "unhandled option"
    return "", defaults


def main(argv):
    mess, defaults = ParseCommandline(argv)
    if mess:
        print mess
        return -1
    logging.basicConfig(level=defaults.debug_level)
    conf = LoadConfig(defaults.config_file)
    server = Server(conf)
    try:
        while 1:
            server.handle_request()
    except KeyboardInterrupt:
        logging.warning("\nCaught ctr-c, taking down the server")
        logging.warning("Please wait while the remaining streams finish..")
    logging.info("exiting server")
    logging.shutdown()
    return


if __name__ == '__main__':
    sys.exit(main(sys.argv))


