#
# HocusPocus
#
# Copyright (C) Robert Muth <robert at muth dot org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import imp
import logging
import os
import subprocess



class Datablob:
    def __init__(self, **args):
        self.__dict__.update(args)

    def __str__(self):
        return str(self.__dict__)

def MakeBlobRefreshList(url, lst, cur):
    blobs = []
    for i in lst:
        d = Datablob()
        d.url = url  % i
        if i == 0:
            d.tag = "None"
        else:
            d.tag = "%ds" % i
        if i == cur:
            d.cur = 1
        else:
            d.cur = None
        blobs.append(d)
    return blobs

def FileStem(filename):
    if filename.endswith("/"):
        filename = filename[:-1]
    filename = os.path.split(filename)[1]
    base = os.path.splitext(filename)[0]
    base = base.replace("-", "_-_")
    base = base.replace("_", " ")
    base = base.replace(".", " ")
    return base

def MakeHtmlBulletsEmpty(n):
    return "&#9675;" * n

def MakeHtmlBulletsFull(n):
    return "&#9679;" * n

def MakeHtmlSquaresEmpty(n):
    return "&#9633;" * n

def MakeHtmlSquaresFull(n):
    return "&#9632;" * n


def EscapeContents(contents):
    contents = contents.replace("&", "&amp;")
    contents = contents.replace("<", "&lt;")
    contents = contents.replace(">", "&gt;")
    return contents

def RunCommand(cmd):
    try:
        process = subprocess.Popen(cmd,
                                   close_fds=True,
                                   stdin=subprocess.PIPE,
                                   stderr=subprocess.STDOUT,
                                   stdout=subprocess.PIPE)
        logging.info("running pid %d [%s]", process.pid, cmd)
        data = process.stdout.read()
        process.wait()

        return process.returncode, data
    except Exception, err:
        return -1, str(err)

def RunCommandBackground(cmd):
    process = subprocess.Popen(cmd,
                               close_fds=True)
    logging.info("running pid %d [%s]", process.pid, cmd)
    return


LOADED_MODULES = {}

def LoadPythonModule(fname):
    global LOADED_MODULES
    desc =  ('.py', 'U', 1)
    dummy, name = os.path.split(fname)
    name, dummy = os.path.splitext(name)
    logging.info("loading module %s %s", fname, name)
    mod =  imp.load_module(name, open(fname), fname, desc)
    LOADED_MODULES[fname] = mod
    return mod, name

def UnloadAllPythonModules():
    global LOADED_MODULES
    values = LOADED_MODULES.values()
    for v in values: del v
    LOADED_MODULES = {}
    return

